package GitLab::Groups;

use utf8;
use strict;
use warnings;
use vars qw($VERSION);

use GitLab::API;
use Log::Any        qw($log);

our $VERSION = v10.4.1;

my $requests = {
    groups  => {
        method      => "GET",
        path        => "/groups",
        query       => [qw(
            skip_groups
            all_available
            search
            order_by
            sort
            statistics
            owned
        )],
        paginated   => 1,
    },

    subgroups => {
        method      => "GET",
        path        => "/groups/<gid>/subgroups",
        query       => [qw(
            skip_groups
            all_available
            search
            order_by
            sort
            statistics
            owned
        )],
        encode      => [qw(
            gid
        )],
        paginated   => 1,
    },

    group_get_projects => {
        method      => "GET",
        path        => "/groups/<gid>/projects",
        optional    => [qw(
            archived
            visibility
            order_by
            sort
            search
            simple
            owned
            starred
        )],
        encode      => [qw(
            gid
        )],
        paginated   => 1,
    },

    group_details => {
        method      => "GET",
        path        => "/groups/<gid>",
        encode      => [qw(
            gid
        )],
    },

    group_create => {
        method      => "POST",
        path        => "/groups",
        required    => [qw(
            name
            path
        )],
        optional    => [qw(
            description
            visibility
            lfs_enabled
            request_access_enabled
            parent_id
            shared_runners_minutes_limit
        )],
    },

    transfer_project_to_group => {
        method      => "POST",
        path        => "/groups/<gid>/projects/<projid>",
    },

    group_update => {
        method      => "PUT",
        path        => "/groups/<gid>",
        optional    => [qw(
            name
            path
            description
            visibility
            lfs_enabled
            request_access_enabled
        )],
    },

    group_delete    => {
        method      => "DELETE",
        path        => "/groups/<gid>",
    },
};

sub import {
    $log->debug("initializing " . __PACKAGE__);
    while (my ($name, $tmpl) = each(%$requests)) {
        $tmpl->{name} = $name unless exists $tmpl->{name};
        GitLab::API->register($tmpl);
    }
}

1;

__END__

=head1 NAME

GitLab::Groups - implements group API calls

See L<GitLab API -- Groups|http://doc.gitlab.com/ce/api/groups.html> for details and
response formats.

=head1 VERSION

Implements API calls for GitLab CE C<v10.4.1>.

=head1 DESCRIPTION

=head2 Notation

Please see the documentation for the L<GitLab::Users> module.

Note that not all optional arguments are listed.
Please refer to the official documentation for the full list.

=head2 Group CRUD operations

=over

=item L<groups()|https://docs.gitlab.com/ce/api/groups.html#list-groups>

    $groups = $gitlab->groups( [:search] [:all_available] [:owned] );

Returns a list of groups.
An optional parameter C<search> can be used to filter the list by substring match on C<name>.

=item L<subgroups()|https://docs.gitlab.com/ce/api/groups.html#list-a-groups-39-s-subgroups>

    $subgroups = $gitlab->subgroups( :gid );

Returns subgroups of the given group.
Accepts same filtering parameters as L</groups()>.

=item L<group_details()|https://docs.gitlab.com/ce/api/groups.html#details-of-a-group>

    $details = $gitlab->group_details( :gid );

Returns details about a group with the given C<gid>.

=item L<group_create()|https://docs.gitlab.com/ce/api/groups.html#new-group>

    $group = $gitlab->group_create( :name, :path, [:description], [:visibility] );

Creates a new group. Required arguments are

    name                the name of the group,
    path                the path for the group (must be unique)

In addition, the following I<optional> arguments can be specified:

    description         group's description
    visibility          see GitLab::API for possible values

=item L<group_update()|https://docs.gitlab.com/ce/api/groups.html#update-group>

    $group = $gitlab->group_update( :gid );

Updates the grup with the given C<gid>.
Takes same arguments as C<group_create>, except they are all optional.

=item L<group_delete()|https://docs.gitlab.com/ce/api/groups.html#remove-group>

    $gitlab->group_delete( :gid );

Deletes the group with the given C<gid>.

=back

=head2 Group members

This section was moved to a separate module L<GitLab::Members>.

=head2 Group projects and transfer

=over

=item L<group_get_projects()|https://docs.gitlab.com/ce/api/groups.html#list-a-group-39-s-projects>

    $projects = $gitlab->group_get_projects( :gid );

Returns a list of projects owned by the group with the given C<gid>.

=item L<transfer_project_to_group()|https://docs.gitlab.com/ce/api/groups.html#transfer-project-to-group>

    $gitlab->transfer_project_to_group( :gid, :projid );

Moves the project with C<projid> to the namespace of the group with the given C<gid>.

I<This method is available only for administrators.>

=back

=head1 AUTHOR

Roman Lacko <L<xlacko1@fi.muni.cz>>

=head1 SEE ALSO

=over

=item L<GitLab>

Wrapper around L<GitLab::API> and other C<GitLab::*> modules.

=back
