package GitLab::Members;

use utf8;
use strict;
use warnings;
use vars qw($VERSION);

use GitLab::API;
use Log::Any        qw($log);

our $VERSION = v10.4.1;

my $requests = {
    "<ENTITY>_members"   => {
        method      => "GET",
        path        => "/<ENTITY>s/<id>/members",
        paginated   => 1,
        optional    => [qw(
            query
        )],
    },

    "<ENTITY>_get_member" => {
        method      => "GET",
        path        => "/<ENTITY>s/<id>/members/<user_id>",
    },

    "<ENTITY>_add_member" => {
        method      => "POST",
        path        => "/<ENTITY>s/<id>/members",
        required    => [qw(
            user_id
            access_level
        )],
        optional    => [qw(
            expires_at
        )],
    },

    "<ENTITY>_update_member" => {
        method      => "PUT",
        path        => "/<ENTITY>s/<id>/members/<user_id>",
        required    => [qw(
            access_level
        )],
        optional    => [qw(
            expires_at
        )],
    },

    "<ENTITY>_update_member" => {
        method      => "PUT",
        path        => "/<ENTITY>s/<id>/members/<user_id>",
        required    => [qw(
            access_level
        )],
        optional    => [qw(
            expires_at
        )],
    },

    "<ENTITY>_delete_member" => {
        method      => "DELETE",
        path        => "/<ENTITY>s/<id>/members/<user_id>",
    },
};

sub import {
    $log->debug("initializing " . __PACKAGE__);

    my $emap = {
        group   => "gid",
        project => "id",
    };

    while (my ($name, $tmpl) = each(%$requests)) {
        foreach my $entity (keys %$emap) {
            my $copy = { %$tmpl };

            $copy->{name} = $name
                unless exists $copy->{name};

            foreach my $v (values %$copy) {
                $v =~ s/<ENTITY>/$entity/g;
                $v =~ s/<id>/<$emap->{$entity}>/g;
            }

            GitLab::API->register($copy);
        }
    }
}

1;

__END__

=head1 NAME

GitLab::Members - implements group and project members API calls

See L<GitLab API -- Group and Project Members|https://docs.gitlab.com/ce/api/members.html> for details and
response formats.

=head1 VERSION

Implements API calls for GitLab CE C<v10.4.1>.

=head1 DESCRIPTION

=head2 Notation

Please see the documentation for the L<GitLab::Users> module.

Note that not all optional arguments are listed.
Please refer to the official documentation for the full list.

=head2 Group and Project Members

=over

=item L<group_members()|https://docs.gitlab.com/ce/api/members.html#list-all-members-of-a-group-or-project>

=item L<project_members()|https://docs.gitlab.com/ce/api/members.html#list-all-members-of-a-group-or-project>

    $members = $gitlab->group_members( :gid );
    $members = $gitlab->project_members( :id );

Returns a list of members of the namespace specified by C<gid> or C<id>.

=item L<group_get_member()|https://docs.gitlab.com/ce/api/members.html#get-a-member-of-a-group-or-project>

=item L<project_get_member()|https://docs.gitlab.com/ce/api/members.html#get-a-member-of-a-group-or-project>

    $gitlab->group_get_member( :gid, :user_id )
    $gitlab->project_get_member( :id, :user_id )

Returns the information about a member of the group or project.
Can also be used to check whether the given user is a member of the group if used with C<< -immortal => 1 >>:

    if (defined $gitlab->group_get_member(gid => $gid, $user_id => $user_id, -immortal => 1)) {
        # is a member
    } else {
        # not a member
    }

=item L<group_add_member()|https://docs.gitlab.com/ce/api/members.html#add-a-member-to-a-group-or-project>

=item L<project_add_member()|https://docs.gitlab.com/ce/api/members.html#add-a-member-to-a-group-or-project>

    $gitlab->group_add_member( :gid, :user_id, :access_level, [:expires_at] );
    $gitlab->project_add_member( :id, :user_id, :access_level, [:expires_at] );

Adds a user with C<user_id> to the group with the given C<gid> with the specified C<access_level>.
Similarly for the project API.
See L<GitLab::API> module for possible access level values.
The optional argument C<expires_at> expects format C<YYYY-MM-DD>.

=item L<group_update_member()|https://docs.gitlab.com/ce/api/members.html#edit-a-member-of-a-group-or-project>

=item L<project_update_member()|https://docs.gitlab.com/ce/api/members.html#edit-a-member-of-a-group-or-project>

    $gitlab->group_update_member( :gid, :user_id, :access_level, [:expires_at] );
    $gitlab->project_update_member( :id, :user_id, :access_level, [:expires_at] );

For user with the given C<user_id> the function changes his C<access_level> in the group with the specified C<gid>.
It can also remove the C<expires_at> property if set to C<undef>.

=item L<group_delete_member()|https://docs.gitlab.com/ce/api/members.html#remove-a-member-from-a-group-or-project>

=item L<project_delete_member()|https://docs.gitlab.com/ce/api/members.html#remove-a-member-from-a-group-or-project>

    $gitlab->group_delete_member( :gid, :user_id );
    $gitlab->project_delete_member( :id, :user_id );

Removes the user with C<user_id> from the group with the specified C<gid>
or a project specidied by the C<id>.

=back

=head1 AUTHOR

Roman Lacko <L<xlacko1@fi.muni.cz>>

=head1 SEE ALSO

=over

=item L<GitLab>

Wrapper around L<GitLab::API> and other C<GitLab::*> modules.

=back
